@@ -1,10 +1,12 @@
 /*
  * MUSCLE SmartCard Development ( http://www.linuxnet.com )
  *
- * Copyright (C) 1999
+ * Copyright (C) 1999-2002
  *  David Corcoran <corcoran@linuxnet.com>
+ * Copyright (C) 2002-2009
+ *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: atrhandler.h 4291 2009-07-01 12:23:22Z rousseau $
+ * $Id: atrhandler.h 5434 2010-12-08 14:13:21Z rousseau $
  */
 
 /**
@@ -16,18 +18,13 @@
 #ifndef __atrhandler_h__
 #define __atrhandler_h__
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
-
 #define SCARD_CONVENTION_DIRECT  0x0001
 #define SCARD_CONVENTION_INVERSE 0x0002
 
-	typedef struct _SMARTCARD_EXTENSION
+	typedef struct
 	{
 
-		struct _ATR
+		struct
 		{
 			int Length;
 			int HistoryLength;
@@ -36,7 +33,7 @@ extern "C"
 		}
 		ATR;
 
-		struct _CardCapabilities
+		struct
 		{
 			UCHAR AvailableProtocols;
 			UCHAR CurrentProtocol;
@@ -44,17 +41,13 @@ extern "C"
 		}
 		CardCapabilities;
 	}
-	SMARTCARD_EXTENSION, *PSMARTCARD_EXTENSION;
+	SMARTCARD_EXTENSION;
 
 	/*
 	 * Decodes the ATR and fills the structure
 	 */
 
-	short ATRDecodeAtr(/*@out@*/ PSMARTCARD_EXTENSION psExtension,
+	short ATRDecodeAtr(/*@out@*/ SMARTCARD_EXTENSION *psExtension,
 		PUCHAR pucAtr, DWORD dwLength);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif							/* __atrhandler_h__ */
