@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: debuglog.c 5047 2010-06-29 14:39:24Z rousseau $
+ * $Id: debuglog.c 5713 2011-05-05 09:25:14Z rousseau $
  */
 
 /**
@@ -51,11 +51,6 @@ void log_xxd(const int priority, const char *msg, const unsigned char *buffer,
 	(void)len;
 }
 
-void DebugLogSuppress(const int lSType)
-{
-	(void)lSType;
-}
-
 void DebugLogSetLogType(const int dbgtype)
 {
 	(void)dbgtype;
@@ -89,7 +84,6 @@ INTERNAL void DebugLogCategory(const int category, const unsigned char *buffer,
  */
 #define DEBUG_BUF_SIZE 2048
 
-static char LogSuppress = DEBUGLOG_LOG_ENTRIES;
 static char LogMsgType = DEBUGLOG_NO_DEBUG;
 static char LogCategory = DEBUG_CATEGORY_NOTHING;
 
@@ -105,8 +99,7 @@ void log_msg(const int priority, const char *fmt, ...)
 	char DebugBuffer[DEBUG_BUF_SIZE];
 	va_list argptr;
 
-	if ((LogSuppress != DEBUGLOG_LOG_ENTRIES)
-		|| (priority < LogLevel) /* log priority lower than threshold? */
+	if ((priority < LogLevel) /* log priority lower than threshold? */
 		|| (DEBUGLOG_NO_DEBUG == LogMsgType))
 		return;
 
@@ -123,14 +116,31 @@ static void log_line(const int priority, const char *DebugBuffer)
 		syslog(LOG_INFO, "%s", DebugBuffer);
 	else
 	{
+		static struct timeval last_time = { 0, 0 };
+		struct timeval new_time = { 0, 0 };
+		struct timeval tmp;
+		int delta;
+
+		gettimeofday(&new_time, NULL);
+		if (0 == last_time.tv_sec)
+			last_time = new_time;
+
+		tmp.tv_sec = new_time.tv_sec - last_time.tv_sec;
+		tmp.tv_usec = new_time.tv_usec - last_time.tv_usec;
+		if (tmp.tv_usec < 0)
+		{
+			tmp.tv_sec--;
+			tmp.tv_usec += 1000000;
+		}
+		if (tmp.tv_sec < 100)
+			delta = tmp.tv_sec * 1000000 + tmp.tv_usec;
+		else
+			delta = 99999999;
+
 		if (LogDoColor)
 		{
 			const char *color_pfx = "", *color_sfx = "\33[0m";
 			const char *time_pfx = "\33[36m", *time_sfx = color_sfx;
-			static struct timeval last_time = { 0, 0 };
-			struct timeval new_time = { 0, 0 };
-			struct timeval tmp;
-			int delta;
 
 			switch (priority)
 			{
@@ -152,28 +162,14 @@ static void log_line(const int priority, const char *DebugBuffer)
 					break;
 			}
 
-			gettimeofday(&new_time, NULL);
-			if (0 == last_time.tv_sec)
-				last_time = new_time;
-
-			tmp.tv_sec = new_time.tv_sec - last_time.tv_sec;
-			tmp.tv_usec = new_time.tv_usec - last_time.tv_usec;
-			if (tmp.tv_usec < 0)
-			{
-				tmp.tv_sec--;
-				tmp.tv_usec += 1000000;
-			}
-			if (tmp.tv_sec < 100)
-				delta = tmp.tv_sec * 1000000 + tmp.tv_usec;
-			else
-				delta = 99999999;
-
-			fprintf(stderr, "%s%.8d%s %s%s%s\n", time_pfx, delta, time_sfx,
+			printf("%s%.8d%s %s%s%s\n", time_pfx, delta, time_sfx,
 				color_pfx, DebugBuffer, color_sfx);
 			last_time = new_time;
 		}
 		else
-			fprintf(stderr, "%s\n", DebugBuffer);
+		{
+			printf("%.8d %s\n", delta, DebugBuffer);
+		}
 	}
 } /* log_msg */
 
@@ -206,36 +202,32 @@ static void log_xxd_always(const int priority, const char *msg,
 void log_xxd(const int priority, const char *msg, const unsigned char *buffer,
 	const int len)
 {
-	if ((LogSuppress != DEBUGLOG_LOG_ENTRIES)
-		|| (priority < LogLevel) /* log priority lower than threshold? */
+	if ((priority < LogLevel) /* log priority lower than threshold? */
 		|| (DEBUGLOG_NO_DEBUG == LogMsgType))
 		return;
 
 	log_xxd_always(priority, msg, buffer, len);
 } /* log_xxd */
 
-void DebugLogSuppress(const int lSType)
-{
-	LogSuppress = lSType;
-}
-
 void DebugLogSetLogType(const int dbgtype)
 {
 	switch (dbgtype)
 	{
 		case DEBUGLOG_NO_DEBUG:
 		case DEBUGLOG_SYSLOG_DEBUG:
-		case DEBUGLOG_STDERR_DEBUG:
+		case DEBUGLOG_STDOUT_DEBUG:
+		case DEBUGLOG_STDOUT_COLOR_DEBUG:
 			LogMsgType = dbgtype;
 			break;
 		default:
-			Log2(PCSC_LOG_CRITICAL, "unknown log type (%d), using stderr",
+			Log2(PCSC_LOG_CRITICAL, "unknown log type (%d), using stdout",
 				dbgtype);
-			LogMsgType = DEBUGLOG_STDERR_DEBUG;
+			LogMsgType = DEBUGLOG_STDOUT_DEBUG;
 	}
 
-	/* log to stderr and stderr is a tty? */
-	if (DEBUGLOG_STDERR_DEBUG == LogMsgType && isatty(fileno(stderr)))
+	/* log to stdout and stdout is a tty? */
+	if ((DEBUGLOG_STDOUT_DEBUG == LogMsgType && isatty(fileno(stdout)))
+		|| (DEBUGLOG_STDOUT_COLOR_DEBUG == LogMsgType))
 	{
 		const char *terms[] = { "linux", "xterm", "xterm-color", "Eterm", "rxvt", "rxvt-unicode" };
 		char *term;
@@ -331,8 +323,7 @@ void debug_msg(const char *fmt, ...)
 	char DebugBuffer[DEBUG_BUF_SIZE];
 	va_list argptr;
 
-	if ((LogSuppress != DEBUGLOG_LOG_ENTRIES)
-		|| (DEBUGLOG_NO_DEBUG == LogMsgType))
+	if (DEBUGLOG_NO_DEBUG == LogMsgType)
 		return;
 
 	va_start(argptr, fmt);
@@ -342,7 +333,7 @@ void debug_msg(const char *fmt, ...)
 	if (DEBUGLOG_SYSLOG_DEBUG == LogMsgType)
 		syslog(LOG_INFO, "%s", DebugBuffer);
 	else
-		fprintf(stderr, "%s\n", DebugBuffer);
+		puts(DebugBuffer);
 } /* debug_msg */
 
 void debug_xxd(const char *msg, const unsigned char *buffer, const int len);
