@@ -3,10 +3,10 @@
  *
  * Copyright (C) 2000-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: eventhandler.c 5567 2011-01-29 13:18:49Z rousseau $
+ * $Id: eventhandler.c 5769 2011-05-30 09:22:57Z rousseau $
  */
 
 /**
@@ -429,7 +429,7 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 				timeout = PCSCLITE_STATUS_EVENT_TIMEOUT;
 
 			ret = rContext->pthCardEvent(rContext->slot, timeout);
-			if (IFD_NO_SUCH_DEVICE == ret)
+			if (IFD_SUCCESS != ret)
 				(void)SYS_USleep(PCSCLITE_STATUS_POLL_RATE);
 		}
 		else
