@@ -5,10 +5,10 @@
  *  David Corcoran <corcoran@linuxnet.com>
  * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: ifdwrapper.c 5567 2011-01-29 13:18:49Z rousseau $
+ * $Id: ifdwrapper.c 5805 2011-06-22 07:06:50Z rousseau $
  */
 
 /**
@@ -47,7 +47,6 @@ LONG IFDSetPTS(READER_CONTEXT * rContext, DWORD dwProtocol, UCHAR ucFlags,
 	UCHAR ucPTS1, UCHAR ucPTS2, UCHAR ucPTS3)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
-	UCHAR ucValue[1];
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	RESPONSECODE(*IFDH_set_protocol_parameters) (DWORD, DWORD, UCHAR,
@@ -68,8 +67,6 @@ LONG IFDSetPTS(READER_CONTEXT * rContext, DWORD dwProtocol, UCHAR ucFlags,
 	 * Error returned by CCID driver is: CCID_Receive Procedure byte conflict
 	 */
 
-	ucValue[0] = rContext->slot;
-
 #ifndef PCSCLITE_STATIC_DRIVER
 	rv = (*IFDH_set_protocol_parameters) (rContext->slot,
 		dwProtocol, ucFlags, ucPTS1, ucPTS2, ucPTS3);
@@ -249,7 +246,6 @@ LONG IFDPowerICC(READER_CONTEXT * rContext, DWORD dwAction,
 {
 	RESPONSECODE rv;
 	DWORD dwStatus;
-	UCHAR ucValue[1];
 	UCHAR dummyAtr[MAX_ATR_SIZE];
 	DWORD dummyAtrLen = sizeof(dummyAtr);
 
@@ -261,7 +257,6 @@ LONG IFDPowerICC(READER_CONTEXT * rContext, DWORD dwAction,
 	 * Zero out everything
 	 */
 	dwStatus = 0;
-	ucValue[0] = 0;
 
 	if (NULL == pucAtr)
 		pucAtr = dummyAtr;
