@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcscdaemon.c 5525 2011-01-18 16:02:25Z rousseau $
+ * $Id: pcscdaemon.c 5764 2011-05-18 11:57:51Z rousseau $
  */
 
 /**
@@ -152,6 +152,7 @@ int main(int argc, char **argv)
 	static struct option long_options[] = {
 		{"config", 1, NULL, 'c'},
 		{"foreground", 0, NULL, 'f'},
+		{"color", 0, NULL, 'T'},
 		{"help", 0, NULL, 'h'},
 		{"version", 0, NULL, 'v'},
 		{"apdu", 0, NULL, 'a'},
@@ -168,7 +169,7 @@ int main(int argc, char **argv)
 		{NULL, 0, NULL, 0}
 	};
 #endif
-#define OPT_STRING "c:fdhvaeCHt:r:s:x"
+#define OPT_STRING "c:fTdhvaeCHt:r:s:x"
 
 	newReaderConfig = NULL;
 	setToForeground = FALSE;
@@ -224,10 +225,15 @@ int main(int argc, char **argv)
 
 			case 'f':
 				setToForeground = TRUE;
-				/* debug to stderr instead of default syslog */
-				DebugLogSetLogType(DEBUGLOG_STDERR_DEBUG);
+				/* debug to stdout instead of default syslog */
+				DebugLogSetLogType(DEBUGLOG_STDOUT_DEBUG);
 				Log1(PCSC_LOG_INFO,
-					"pcscd set to foreground with debug send to stderr");
+					"pcscd set to foreground with debug send to stdout");
+				break;
+
+			case 'T':
+				DebugLogSetLogType(DEBUGLOG_STDOUT_COLOR_DEBUG);
+				Log1(PCSC_LOG_INFO, "Force colored logs");
 				break;
 
 			case 'd':
@@ -260,8 +266,8 @@ int main(int argc, char **argv)
 				break;
 
 			case 'H':
-				/* debug to stderr instead of default syslog */
-				DebugLogSetLogType(DEBUGLOG_STDERR_DEBUG);
+				/* debug to stdout instead of default syslog */
+				DebugLogSetLogType(DEBUGLOG_STDOUT_DEBUG);
 				HotPlug = TRUE;
 				break;
 
@@ -459,8 +465,8 @@ int main(int argc, char **argv)
 	 * signal_trap() does just set a global variable used by the main loop
 	 */
 	(void)signal(SIGQUIT, signal_trap);
-	(void)signal(SIGTERM, signal_trap);
-	(void)signal(SIGINT, signal_trap);
+	(void)signal(SIGTERM, signal_trap); /* default kill signal & init round 1 */
+	(void)signal(SIGINT, signal_trap);	/* sent by Ctrl-C */
 
 	/* exits on SIGALARM to allow pcscd to suicide if not used */
 	(void)signal(SIGALRM, signal_trap);
@@ -639,7 +645,7 @@ static void at_exit(void)
 		close(pipefd[1]);
 	}
 
-	_exit(ExitValue);
+	exit(ExitValue);
 }
 
 static void clean_temp_files(void)
@@ -675,6 +681,14 @@ static void signal_trap(int sig)
 {
 	Log2(PCSC_LOG_INFO, "Received signal: %d", sig);
 
+	/* do not wait if asked to terminate
+	 * avoids waiting after the reader(s) in shutdown for example */
+	if (SIGTERM == sig)
+	{
+		Log1(PCSC_LOG_INFO, "Direct suicide");
+		at_exit();
+	}
+
 	/* the signal handler is called several times for the same Ctrl-C */
 	if (AraKiri == FALSE)
 	{
@@ -709,7 +723,7 @@ static void print_version (void)
 {
 	printf("%s version %s.\n",  PACKAGE, VERSION);
 	printf("Copyright (C) 1999-2002 by David Corcoran <corcoran@linuxnet.com>.\n");
-	printf("Copyright (C) 2001-2010 by Ludovic Rousseau <ludovic.rousseau@free.fr>.\n");
+	printf("Copyright (C) 2001-2011 by Ludovic Rousseau <ludovic.rousseau@free.fr>.\n");
 	printf("Copyright (C) 2003-2004 by Damien Sauveron <sauveron@labri.fr>.\n");
 	printf("Report bugs to <muscle@lists.musclecard.com>.\n");
 
@@ -724,7 +738,8 @@ static void print_usage (char const * const progname)
 	printf("  -a, --apdu		log APDU commands and results\n");
 	printf("  -c, --config		path to reader.conf\n");
 	printf("  -f, --foreground	run in foreground (no daemon),\n");
-	printf("			send logs to stderr instead of syslog\n");
+	printf("			send logs to stdout instead of syslog\n");
+	printf("  -T, --color       force use of colored logs\n");
 	printf("  -h, --help		display usage information\n");
 	printf("  -H, --hotplug		ask the daemon to rescan the available readers\n");
 	printf("  -v, --version		display the program version number\n");
@@ -736,10 +751,12 @@ static void print_usage (char const * const progname)
 	printf("  -t, --max-thread	maximum number of threads (default %d)\n", PCSC_MAX_CONTEXT_THREADS);
 	printf("  -s, --max-card-handle-per-thread	maximum number of card handle per thread (default: %d)\n", PCSC_MAX_CONTEXT_CARD_HANDLES);
 	printf("  -r, --max-card-handle-per-reader	maximum number of card handle per reader (default: %d)\n", PCSC_MAX_READER_HANDLES);
+	printf("  -x, --auto-exits	pcscd will quit after %d seconds of inactivity\n", TIME_BEFORE_SUICIDE);
 #else
 	printf("  -a    log APDU commands and results\n");
 	printf("  -c 	path to reader.conf\n");
-	printf("  -f	run in foreground (no daemon), send logs to stderr instead of syslog\n");
+	printf("  -f	run in foreground (no daemon), send logs to stdout instead of syslog\n");
+	printf("  -T    force use of colored logs\n");
 	printf("  -d 	display debug messages. Output may be:\n");
 	printf("  -h 	display usage information\n");
 	printf("  -H	ask the daemon to rescan the available readers\n");
@@ -747,6 +764,7 @@ static void print_usage (char const * const progname)
 	printf("  -t    maximum number of threads\n");
 	printf("  -s    maximum number of card handle per thread\n");
 	printf("  -r    maximum number of card handle per reader\n");
+	printf("  -x	pcscd will quit after %d seconds of inactivity\n", TIME_BEFORE_SUICIDE);
 #endif
 }
 
