@@ -8,7 +8,7 @@
  * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.h 5434 2010-12-08 14:13:21Z rousseau $
+ * $Id: winscard_msg.h 5796 2011-06-16 08:56:03Z rousseau $
  */
 
 /**
@@ -258,7 +258,7 @@
 	int32_t ClientSetupSession(uint32_t *);
 	int32_t ClientCloseSession(uint32_t);
 	LONG MessageReceiveTimeout(uint32_t command, /*@out@*/ void *buffer,
-		uint64_t buffer_size, int32_t filedes, int32_t timeOut);
+		uint64_t buffer_size, int32_t filedes, unsigned long timeOut);
 	LONG MessageSendWithHeader(uint32_t command, uint32_t dwClientID,
 		uint64_t size, void *data);
 #endif
